@@ -1,9 +1,11 @@
 __author__ = 'chimezieogbuji'
 
+import memcache,hashlib
 from wheezy.http.response import HTTPResponse
 from wheezy.http.middleware import http_cache_middleware_factory
 from wheezy.http.cache import CacheableResponse, NotModifiedResponse
 from wheezy.caching import CacheDependency
+from wheezy.caching.memcache import client_factory
 from wheezy.http import bootstrap_http_defaults, CacheProfile, WSGIApplication
 from wheezy.caching import MemoryCache
 from wheezy.http import HTTPCachePolicy
@@ -15,11 +17,38 @@ from wheezy.http.cacheprofile import RequestVary
 #Memory cache (local to akara process)
 #cache = MemoryCache()
 
-#memcache (distributed)
-from wheezy.caching.memcache import client_factory
-cache = client_factory(['unix:/tmp/memcached.sock'])
+class memcached(object):
+    """
+    Decorator that uses memcache to cache response to
+    calling the function where the key is based on
+    the values of the arguments given
 
-cache_factory = lambda: cache
+    modified from http://www.zieglergasse.at/blog/2011/python/memcached-decorator-for-python/
+    """
+    def __init__(self,memcache_socket):
+        self.memcache_socket = memcache_socket
+
+    def __call__(self, f):
+
+        def newfn(*args, **kwargs):
+            mc = memcache.Client([self.memcache_socket], debug=0)
+            # generate md5 out of args and function
+            m = hashlib.md5()
+            margs = [x.__repr__() for x in args]
+            mkwargs = [x.__repr__() for x in kwargs.values()]
+            map(m.update, margs + mkwargs)
+            m.update(f.__name__)
+            m.update(f.__class__.__name__)
+            key = m.hexdigest()
+
+            value = mc.get(key)
+            if value:
+                return value
+            else:
+                value = f(*args, **kwargs)
+                mc.set(key, value, 60)
+                return value
+        return newfn
 
 class WheezyCachingAdapterSetup(object):
     def __init__(self,
@@ -30,7 +59,8 @@ class WheezyCachingAdapterSetup(object):
                  environ = None,
                  debug = False,
                  asFn = False,
-                 ttl=15):
+                 ttl=15,
+                 memcache_socket = 'unix:/tmp/memcached.sock',):
         """
         Used as an Akara WSGI wrapper, i.e.:
         @simple_service(...,wsgi_wrapper=WheezyCachingAdapterSetup( ... ))
@@ -78,6 +108,7 @@ class WheezyCachingAdapterSetup(object):
             - http://packages.python.org/wheezy.caching/userguide.html#cachedependency
 
         """
+        cache           = client_factory([memcache_socket])
         self.ttl        = ttl
         self.asFn       = asFn
         self.debug      = debug
@@ -88,8 +119,9 @@ class WheezyCachingAdapterSetup(object):
         self._middlewareFactories = [WheezyAkaraMiddleWareFactory]
         self._options = options if options else {
             'ENCODING': 'utf-8',
-            'http_cache_factory': cache_factory
+            'http_cache_factory': lambda: cache
         }
+        self._options['cache']          = cache
         if dependency:
             self._options['dependency'] = dependency
         if noCache:
@@ -146,6 +178,33 @@ class WheezyAkaraMiddleWareFactory(object):
         self.middleware_vary = middleware_vary
         self.profiles = global_profiles
 
+    def run_app(self, request, response, start_response,environ,default_policy=None):
+        if self.debug:
+            print "Invoking application"
+        result = self.akara_application(request.environ, start_response)
+        try:
+            response.buffer.extend(result)
+        finally:
+            if hasattr(result, 'close'):
+                result.close()
+        from amara.xslt.result import stringresult
+
+        def normalizeResult(item):
+            if isinstance(item, stringresult):
+                return unicode(item)#._stream.getvalue()
+            else:
+                return item
+
+        response.buffer = map(normalizeResult, response.buffer)
+        policy = default_policy
+        if 'wheezy.http.cache_policy' in environ:
+            policy = environ['wheezy.http.cache_policy']
+            if self.debug:
+                print "Policy set in akara app"
+        elif self.debug:
+            print "Policy not set in akara app!"
+        response.cache_policy = policy
+
     def __call__(self, request, following):
         """
         Combination of (in order):
@@ -156,7 +215,7 @@ class WheezyAkaraMiddleWareFactory(object):
             if self.debug:
                 print "invalidating cache: ", cacheName
             dependency = CacheDependency(cacheName)
-            dependency.delete(cache_factory())
+            dependency.delete(self.options['cache'])
 
         assert not following
         request.environ['wheezy.http.HTTPCachePolicy'] = HTTPCachePolicy
@@ -169,51 +228,19 @@ class WheezyAkaraMiddleWareFactory(object):
             response.headers = [(name, value) for name, value in headers
                                               if name != 'Content-Length']
             return response.write_bytes
-        result = self.akara_application(request.environ, start_response)
-        try:
-            response.buffer.extend(result)
-        finally:
-            if hasattr(result, 'close'):  # pragma: nocover
-                result.close()
-
-        from amara.xslt.result import stringresult
-        def normalizeResult(item):
-            if isinstance(item,stringresult):
-                return unicode(item)#._stream.getvalue()
-            else:
-                return item
-
-        response.buffer = map(normalizeResult,response.buffer)
         environ = request.environ
-        policy = None
-        if 'wheezy.http.cache_policy' in environ:
-            policy = environ['wheezy.http.cache_policy']
-            response.cache_policy = policy
-
         if 'noCache' in self.options:
             response.cache_profile = CacheProfile('none', no_store=True)
         else:
             response.cache_profile = CacheProfile('public', duration=timedelta(minutes=self.options['ttl']))
 
-        assert (policy or
-                'wheezy.http.noCache' in environ or
-                'noCache' in self.options or
-                response.status_code != 200)
-        dependency = self.options.get(
-            'dependency',
-            environ.get('wheezy.http.cache_dependency'))
-        if self.debug:
-            print "Cache name: (for dependencies) %s"%dependency if dependency else "No cache name given!"
-        if dependency:
-            response.dependency = CacheDependency(dependency)
-
         _response = response
 
         middleware_key = 'C' + self.middleware_vary.key(request)
         _cache_profile = self.profiles[middleware_key]
         if _cache_profile:
             request_key = _cache_profile.request_vary.key(request)
-            context = cache_factory()
+            context = self.options['cache']
             cache = context.__enter__()
             try:
                 response = cache.get(request_key, _cache_profile.namespace)
@@ -221,7 +248,7 @@ class WheezyAkaraMiddleWareFactory(object):
                 context.__exit__(None, None, None)
             if response and _response.status_code == 200:  # cache hit
                 if self.debug:
-                    print "Cache hit!", request_key
+                    print "Cache hit!", request_key, request
                 environ = request.environ
                 if response.etag:
                     none_match = environ.get('HTTP_IF_NONE_MATCH', None)
@@ -238,7 +265,23 @@ class WheezyAkaraMiddleWareFactory(object):
             else:
                 response = _response
                 if self.debug:
-                    print "No cache hit"
+                    print "No cache hit, Akara app"
+                self.run_app(
+                    request,
+                    response,
+                    start_response,
+                    environ,
+                    response.cache_profile.cache_policy())
+        else:
+            if self.debug:
+                print "No cache profile, running Akara app"
+            self.run_app(
+                request,
+                response,
+                start_response,
+                environ,
+                response.cache_profile.cache_policy())
+
         if response.status_code == 200:
             response_cache_profile = response.cache_profile
             if 'noCache' not in self.options and response_cache_profile:
@@ -246,14 +289,24 @@ class WheezyAkaraMiddleWareFactory(object):
                     self.profiles[middleware_key] = response_cache_profile
                     request_key = response_cache_profile.request_vary.key(
                         request)
-                dependency = response.dependency
+                #Check dependencies set by run application
+                dependency = self.options.get(
+                    'dependency',
+                    environ.get('wheezy.http.cache_dependency'))
+                if self.debug:
+                    print "Cache name: (for dependencies) %s"%dependency if dependency else "No cache name given!"
+                if dependency:
+                    print "Dependency key: ", dependency
+                    response.dependency_key = CacheDependency(dependency)
+
+                dependency = response.dependency_key
                 response = CacheableResponse(response)
-                context = cache_factory()
+                context = self.options['cache']
                 cache = context.__enter__()
                 try:
                     if dependency:
                         if self.debug:
-                            print "Attempting to set cache (with dependency)"
+                            print "Attempting to set cache (with dependency)", dependency
                         cache.set_multi({
                             request_key: response,
                             dependency.next_key(
