@@ -1,14 +1,13 @@
 __author__ = 'chimezieogbuji'
 
 import cgi, inspect, re
-from cStringIO   import StringIO
+from cStringIO   import StringIO as c_string_io
+from StringIO import StringIO as regular_string_io
 from amara.xslt  import transform
 from akara       import request, response, global_config
-from Ft.Xml.Xslt import Processor
-from Ft.Xml      import InputSource
-from Ft.Lib      import Uri
-
-from akamu.xslt.extensions import NS
+from amara.xslt.processor import processor as amara_processor
+from amara.lib import inputsource
+# from akamu.xslt.extensions import NS
 
 FIREFOX_PATTERN = re.compile(r'15\..+')
 IE_9_PATTERN    = re.compile(r'9.0.+')
@@ -37,22 +36,27 @@ def ClientSideXSLTCapable(environ):
                        CLIENT_SIDE_BROWSERS)
     return bool(supported)
 
-def TransformWithAkamuExtensions(src,xform,manager,params=None,baseUri=NS):
-    params = params if params else {}
-    processor         = Processor.Processor()
+def TransformWithAkamuExtensions(src,xform,manager,params=None,baseUri=None):
+    import warnings;warnings.warn("Akamu XSLT extensions are not fully supported!")
+    baseUri = baseUri if baseUri else NS
+    from amara.xslt.result import streamresult
+    from amara.xpath.util import parameterize
+    params = parameterize(params) if params else {}
+    processor         = amara_processor(ignore_pis=True)#Processor.Processor()
     processor.manager = manager
     processor.registerExtensionModules(['akamu.xslt.extensions'])
 
-    result = StringIO()
-    source = InputSource.DefaultFactory.fromString(src,baseUri)
-    isrc   = InputSource.DefaultFactory.fromString(xform,baseUri)
-    processor.appendStylesheet(isrc)
-    processor.run(
-        source,
-        outputStream=result,
-        ignorePis=True,
-        topLevelParams=params
-    )
+    result = streamresult(StringIO())
+    source = inputsource(src,baseUri)#InputSource.DefaultFactory.fromString(src,baseUri)
+    isrc   = inputsource(xform,baseUri)#InputSource.DefaultFactory.fromString(xform,baseUri)
+    processor.append_transform(isrc)
+    proc.run(source, params, result)
+    # processor.run(
+    #     source,
+    #     result=result,
+    #     ignorePis=True,
+    #     topLevelParams=params
+    # )
     return result.getvalue()
 
 NOOPXML = u'<Root/>'
@@ -66,22 +70,30 @@ class xslt_rest(object):
     the result of the transformation of this (using a user-specified
     transform) is used as the result of the service
     """
-    def __init__(self, transform, source = None, argRemap = None, parameters = None, clientSide = False):
+    def __init__(self,
+                 transform,
+                 source = None,
+                 argRemap = None,
+                 parameters = None,
+                 clientSide = False,
+                 srcIsFn = False):
         self.clientSide = clientSide
-        self.argRemap  = argRemap if argRemap else {}
-        self.transform = transform
-        self.params    = parameters if parameters else {}
-        self.source = source if source else None
+        self.argRemap   = argRemap if argRemap else {}
+        self.transform  = transform
+        self.params     = parameters if parameters else {}
+        self.source     = source if source else None
+        self.srcIsFn    = srcIsFn
 
     def __call__(self, func):
         def innerHandler(*args, **kwds):
             argNames = inspect.getargspec(func).args
-            parameters = self.params
+            parameters = {}
+            parameters.update(self.params)
             isaPost = len(args) == 2 and list(argNames) == ['body','ctype']
             if isaPost:
                 #Parameters in POST body
                 fields = cgi.FieldStorage(
-                    fp=StringIO(args[0]),
+                    fp=regular_string_io(args[0]),
                     environ=request.environ
                 )
                 for k in fields:
@@ -108,19 +120,39 @@ class xslt_rest(object):
             argInfo = inspect.getargspec(func)
             vargs    = argInfo.varargs
             keywords = argInfo.keywords
+
+            #If the source is a function, then the parameters
+            #are given to it, otherwise, to the decorated service function
+            srcFn = self.source if self.srcIsFn else func
+
             if keywords is None and argInfo.defaults:
                 keywords = argInfo.args[-len(argInfo.defaults):]
                 vargs    = argInfo.args[:-len(argInfo.defaults)]
             if isaPost:
-                src = func(*args) if not self.source else self.source
+                src = srcFn(*args) if (
+                    self.source is None or self.srcIsFn
+                ) else self.source
             elif vargs and keywords:
-                src = func(*args, **kwds) if not self.source else self.source
+                src = srcFn(*args, **kwds) if (
+                    self.source is None or self.srcIsFn
+                ) else self.source
             elif vargs:
-                src = func(*args) if not self.source else self.source
+                src = srcFn(*args) if (
+                    self.source is None or self.srcIsFn
+                ) else self.source
             elif keywords:
-                src = func(**kwds) if not self.source else self.source
+                src = srcFn(**kwds) if (
+                    self.source is None or self.srcIsFn
+                ) else self.source
+            elif argInfo.args and parameters:
+                src = srcFn(*map(lambda i:parameters[i],
+                                 argInfo.args)) if (
+                    self.source is None or self.srcIsFn
+                ) else self.source
             else:
-                src = func() if not self.source else self.source
+                src = srcFn() if (
+                    self.source is None or self.srcIsFn
+                ) else self.source
             isInfoResource = (isinstance(response.code,int) and
                               response.code == 200
                              ) or (isinstance(response.code,basestring) and
@@ -147,5 +179,13 @@ class xslt_rest(object):
                     doc.xml_insert(0,pi)
                     return doc
                 else:
-                    return transform(src,self.transform,params=parameters)
+                    rtStream = regular_string_io()
+                    #Unicde must be given as inefficient stream
+                    #See http://docs.python.org/2/library/stringio.html#cStringIO.StringIO
+                    #and limitation on akara transform function
+                    # print src, type(src), self.transform
+                    # src = regular_string_io(src) if isinstance(src,unicode) else c_string_io(src)
+                    src = src.encode('utf-8') if isinstance(src,unicode) else src
+                    transform(src,self.transform,params=parameters,output=rtStream)
+                    return rtStream.getvalue()
         return innerHandler
\ No newline at end of file
